#define F_CPU 8000000
#include <avr/io.h>
#include <util/delay.h>
#define LM75ADDR 0x90 //adres czujnika temperatury (zworki zalozone na A1 i A2 - GND, A0 - bez zworki)

void UartInit(uint32_t baudrate); //inicjalizacja modulu UART ATmegi
void UartPutc(char c); //wyslanie pojedynczego znaku poprzez UART
void UartSend(char* s); //wyslanie ciagu znakow 
void twiinit(uint16_t bps); //inicjalizacja modulu TWI
void twistart(void); // procedura transmisji sygnau START
void twistop(void); // procedura transmisji sygnau STOP
void twiwrite(char data);  //procedura transmisji bajtu danych
char twiread(char ack);  //procedura odczytu bajtu danych
int8_t ReadTemp(void); //procedura pobierania temperatury z czujnika

int8_t ReadTemp(void)
{
	int8_t temp; 
	twistart(); //poczatek transmisji
	twiwrite(LM75ADDR | 0x01); //wyslanie adresu (7 bitow oraz 1 bitu read/write)
	temp = twiread(0); //pobieranie temperatury (8 bitow)
	twistop(); //koniec transmisji
	return temp; //funkcja zwraca temperature
}
 
void UartInit(uint32_t baudrate) 
{
    uint16_t ubrr = F_CPU / 16 / baudrate - 1; //obliczenie odpowiedniej wartosci wspolczynnika UBRR dla zadanego baudrate (szczegoly w nocie katalogowej)
    //rejestr UBRR jest podzielony na dwie czesci po 8 bitow
    UBRRH = (uint8_t)(ubrr >> 8); //zapis 8 bitow (bardziej znaczacych) z obliczonej wczesniej wartosci do rejestru UBRRH
    UBRRL = (uint8_t)ubrr; //zapis kolejnych 8 bitow (mniej znaczacych) do UBRRL
    UCSRB = (1 << TXEN)|(1 << RXEN); //skonfigurowanie odpowiednich pinow ATmegi do pracy jako UART
}

void UartPutc(char c) 
{
	while (!(UCSRA & (1 << UDRE))); //dopoki rejestr jest pelny nie mozna wpisac do niego kolejnego znaku (szczegoly w dokumentacji)
	UDR = c; //przypisanie znaku do UDR - bufora nadawczego UART
}

void UartSend(char* s)
{
	while (*s) //dopoki wskaznik jest niezerowy
		UartPutc(*s++); //przeslij kolejny znak do UDR w celu wyslania poprzez UART
}

void twiinit(uint16_t bps)
{
	TWBR = (F_CPU/(bps) - 16)/2; //czestotliwosc zegara TWI
	TWCR = (1<<TWEN); //wlaczenie TWI
}

void twistart(void)
{
	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN); 
	while (!(TWCR & (1<<TWINT)));
}

void twistop(void)
{
	TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO);
	while ((TWCR & (1<<TWSTO)));
}


void twiwrite(char data)
{
	TWDR = data;  //wpis danych do rejestru wysylajacego
	TWCR = (1<<TWINT) | (1<<TWEN); 
	while (!(TWCR & (1<<TWINT)));
}


char twiread(char ack)
{
	TWCR = ack
		 ? ((1 << TWINT) | (1 << TWEN) | (1 << TWEA))
		 : ((1 << TWINT) | (1 << TWEN)) ;
	while (!(TWCR & (1<<TWINT)));
	return TWDR;
}

int main(void)
{
  int8_t temperature;
  twiinit(100000); //zegar 100kHz
  UartInit(9600); //takie samo baudrate nalezy ustawic w terminalu na komputerze, 8-bitow dlugosci, brak bitu parzystosci, 1 bit stopu
  while(1)
  {
     _delay_ms(3000); //opoznienie okolo 3 sekund pomiedzy pomiarami
	 temperature=ReadTemp(); //odczyt temperatury
     char buff[5]; //bufor przeznaczony do konwersji z typu int8_t do ciagu znakow
     itoa(temperature, buff, 10); //funkcja konwertujaca liczbe na ciag znakow w formacie dziesietnym
     UartSend(buff); //wysylanie temperatury w formie dziesietnej
     UartSend("\r\n"); //poczatek nowej linii
  }
}
