#define F_CPU 8000000
#include <avr/io.h>
#include <util/delay.h>

void UartInit(uint32_t baudrate); //inicjalizacja modulu UART ATmegi
void UartPutc(char c); //wyslanie pojedynczego znaku poprzez UART
void UartSend(char* s); //wyslanie ciagu znakow 

 
void UartInit(uint32_t baudrate) 
{
    uint16_t ubrr = F_CPU / 16 / baudrate - 1; //obliczenie odpowiedniej wartosci wspolczynnika UBRR dla zadanego baudrate (szczegoly w nocie katalogowej)
    //rejestr UBRR jest podzielony na dwie czesci po 8 bitow
    UBRRH = (uint8_t)(ubrr >> 8); //zapis 8 bitow (bardziej znaczacych) z obliczonej wczesniej wartosci do rejestru UBRRH
    UBRRL = (uint8_t)ubrr; //zapis kolejnych 8 bitow (mniej znaczacych) do UBRRL
    UCSRB = (1 << TXEN)|(1 << RXEN); //skonfigurowanie odpowiednich pinow ATmegi do pracy jako UART
}

void UartPutc(char c) 
{
	while (!(UCSRA & (1 << UDRE))); //dopoki rejestr jest pelny nie mozna wpisac do niego kolejnego znaku (szczegoly w dokumentacji)
	UDR = c; //przypisanie znaku do UDR - bufora nadawczego UART
}

void UartSend(char* s)
{
	while (*s) //dopoki wskaznik jest niezerowy
		UartPutc(*s++); //przeslij kolejny znak do UDR w celu wyslania poprzez UART
}

int main(void)
{
  UartInit(9600); //takie samo baudrate nalezy ustawic w terminalu na komputerze, 8-bitow dlugosci, brak bitu parzystosci, 1 bit stopu
  while(1)
  {
     UartSend("http://www.firstmenonthemoon.com/"); //wysylanie ciagu znakow
	 UartSend("\r\n"); //przejscie do nowej linii
	 _delay_ms(1000); //ok. 1 sekunda odstepu pomiedzy wysylaniami
  }
}
